#include <Wire.h>  // Only needed for Arduino 1.6.5 and earlier
#include "SSD1306.h" // alias for `#include "SSD1306Wire.h"`
#include <BME280I2C.h>
#include "images.h"

#define I2C_SDA 4
#define I2C_SCL 15

#define ADC_POT 13

SSD1306  display(0x3c, I2C_SDA, I2C_SCL);
BME280I2C bme;

void vPantallaUnaLinia(String szStr) {
  display.clear();

  display.setTextAlignment(TEXT_ALIGN_LEFT);
  display.setFont(ArialMT_Plain_16);
  display.drawString(0, 22, szStr);
  display.display();
}

void vInitBme280() {
  Wire.begin(I2C_SDA, I2C_SCL);

  while (!bme.begin())
  {
    Serial.println("No es detecta el sensor BME280!");
    vPantallaUnaLinia("No hi ha BME280");
    delay(1000);
  }
  Serial.println("Trobat el sensor BME280!");
  vPantallaUnaLinia("Detectat BME280");
  delay(1000);
  // bme.chipID(); // Deprecated. See chipModel().
  switch (bme.chipModel())
  {
    case BME280::ChipModel_BME280:
      Serial.println("Found BME280 sensor! Success.");
      break;
    case BME280::ChipModel_BMP280:
      Serial.println("Found BMP280 sensor! No Humidity available.");
      break;
    default:
      Serial.println("Found UNKNOWN sensor! Error!");
  }
}

void vInitOLED() {
  display.init();
  display.flipScreenVertically();

  display.clear();
  display.drawXbm(0, 0, 128, 64, logo128x64_bmp);
  display.display();

  delay(1000);
}

void vLecturaBme280(float *pfTemp, float *pfHum, float *pfPres) {
  BME280::TempUnit tempUnit(BME280::TempUnit_Celsius);
  BME280::PresUnit presUnit(BME280::PresUnit_Pa);

  bme.read(*pfPres, *pfTemp, *pfHum, tempUnit, presUnit);
}

void vPresentaPotBme280() {
  float fTemp, fHum, fPres;

  vLecturaBme280(&fTemp, &fHum, &fPres);
  display.clear();
  // Font Demo1
  // create more fonts at http://oleddisplay.squix.ch/
  display.setTextAlignment(TEXT_ALIGN_LEFT);
  display.setFont(ArialMT_Plain_10);
  display.drawString(0, 0, "Temp: " + String(fTemp) + " C");
  display.drawString(0, 14, "Hum: " + String(fHum) + " % HR");
  display.drawString(0, 28, "Pres: " + String(fPres) + " Pa");
  display.setFont(ArialMT_Plain_24);
  display.drawString(10, 42, "Pot: " + String(analogRead(ADC_POT)));

  display.display();
}

void setup() {
  Serial.begin(115200);

  vInitOLED();
  vInitBme280();
}

void drawFontFaceDemo() {
  // Font Demo1
  // create more fonts at http://oleddisplay.squix.ch/
  display.setTextAlignment(TEXT_ALIGN_LEFT);
  display.setFont(ArialMT_Plain_10);
  display.drawString(0, 0, "Tensió: " + String(millis() / 1000));
  display.drawString(0, 10, "Corrent: " + String(millis() / 1000));
  display.setFont(ArialMT_Plain_16);
  display.drawString(0, 22, "Potència: ");
  display.setFont(ArialMT_Plain_24);
  display.drawString(10, 42, "Pot: " + String(analogRead(13)));
}

void loop() {
  vPresentaPotBme280();

  delay(500);
}
